.x-toggle-slide-container,.x-toggle-slide-container label{
    user-select:none;
    -moz-user-select:none;
    -khtml-user-select:none;
}
.x-toggle-slide-disabled{
    filter:alpha(opacity=50);
    -moz-opacity:0.5;
    -khtml-opacity:0.5;
    opacity:0.5;
}
.x-toggle-slide-container{
    position:relative;
    overflow:hidden;
    white-space:nowrap;
    height:23px;
    cursor:pointer;
    overflow:hidden;
}
.x-toggle-slide-container div.holder{
    height:23px;
    overflow:hidden;
    -moz-border-radius:4px;
    -webkit-border-radius:4px;
    -o-border-radius:4px;
    -ms-border-radius:4px;
    -khtml-border-radius:4px;
    border-radius:4px;
}
.x-toggle-slide-container label{
    white-space:nowrap;
    font-size:15px;
    line-height:15px;
    font-weight:bold;
    font-family:"Helvetica Neue",Arial,Helvetica,sans-serif;
    text-transform:uppercase;
    cursor:pointer;
    display:inline-block;
    clear:both;
    -moz-box-shadow:inset -2px 2px 10px -5px black;
    -webkit-box-shadow:inset -2px 2px 10px -5px black;
    box-shadow:inset -2px 2px 10px -5px black;
    height:23px;
    width:auto;
    padding-top:3px;
    overflow:hidden;
}
label.x-toggle-slide-label-on{
    background:#5EA2DA;
    color:#F7FAFD;
    text-shadow:0px 0px 2px #364A5D;
}
label.x-toggle-slide-label-on span{
    padding-left:8px;
}
label.x-toggle-slide-label-off{
    background:#D5D6D6;
    color:#929496;
    text-shadow:1px 0px 1px #fff;
    text-align:right;
}
label.x-toggle-slide-label-off span{
    padding-right:8px;
}
.x-toggle-slide-thumb{
    display:block;
    height:23px;
    cursor:pointer;
    position:absolute;
    top:0;
    left:0;
    width:0;
    background:rgb(255,255,255);
    background:-moz-linear-gradient(top,rgba(255,255,255,1) 0%,rgba(229,229,229,1) 100%);
    background:-webkit-gradient(linear,left top,left bottom,color-stop(0%,rgba(255,255,255,1)),color-stop(100%,rgba(229,229,229,1)));
    background:-webkit-linear-gradient(top,rgba(255,255,255,1) 0%,rgba(229,229,229,1) 100%);
    background:-o-linear-gradient(top,rgba(255,255,255,1) 0%,rgba(229,229,229,1) 100%);
    background:-ms-linear-gradient(top,rgba(255,255,255,1) 0%,rgba(229,229,229,1) 100%);
    background:linear-gradient(top,rgba(255,255,255,1) 0%,rgba(229,229,229,1) 100%);
    filter:progid:DXImageTransform.Microsoft.gradient( startColorstr='#ffffff',endColorstr='#e5e5e5',GradientType=0 );
    border:1px solid #999999;
    -moz-border-radius:4px;
    -webkit-border-radius:4px;
    -o-border-radius:4px;
    -ms-border-radius:4px;
    -khtml-border-radius:4px;
    border-radius:4px;
    padding-left:3px;
}
.x-item-disabled .x-toggle-slide-label-on span{
    color:white !important;
}
.x-item-disabled .x-toggle-slide-label-off span{
    color:gray !important;
}
.x-toolbar .x-toggle-slide-container label{
    font-size:12px !important;
    height:17px;
    padding-top:1px;
}
.x-toolbar .x-toggle-slide-container{
    height:17px;
}
.x-toolbar .x-toggle-slide-container div.holder{
    height:17px;
    -moz-border-radius:4px;
    -webkit-border-radius:4px;
    -o-border-radius:4px;
    -ms-border-radius:4px;
    -khtml-border-radius:4px;
    border-radius:4px;
}
.x-toolbar label.x-toggle-slide-label-on{
}
.x-toolbar label.x-toggle-slide-label-on span{
    padding-left:6px;
}
.x-toolbar label.x-toggle-slide-label-off{
}
.x-toolbar label.x-toggle-slide-label-off span{
    padding-right:6px;
}
.x-toolbar .x-toggle-slide-thumb{
    height:17px;
}
.x-toolbar-right .x-toggle-slide-label-on{
    text-align:left;
}
.x-field .x-toggle-slide-container label{
    font-size:12px !important;
    height:18px;
    padding-top:1px;
}
.x-field .x-toggle-slide-container{
    height:18px;
}
.x-field .x-toggle-slide-container div.holder{
    height:18px;
}
.x-field label.x-toggle-slide-label-on{
}
.x-field label.x-toggle-slide-label-on span{
    padding-left:6px;
}
.x-field label.x-toggle-slide-label-off{
}
.x-field label.x-toggle-slide-label-off span{
    padding-right:6px;
}
.x-field .x-toggle-slide-thumb{
    height:18px;
}
.x-toolbar-right .x-toggle-slide-label-on{
    text-align:left;
}
